@@ -6,14 +6,13 @@
 #include "../utility/os.hpp"
 #include "../utility/math.hpp"
 #include "../utility/small_vector.hpp"
-#include "../utility/serializer.hpp"
 #include "error.hpp"
 #include "declarations.hpp"
 #include "semaphore.hpp"
 #include "environment.hpp"
 #include "topology.hpp"
 
-/** 
+/**
 @file graph.hpp
 @brief graph include file
 */
@@ -30,9 +29,9 @@ namespace tf {
 class CustomGraphBase {
 
   public:
-  
+
   virtual void dump(std::ostream&, const void*, const std::string&) const = 0;
-  virtual ~CustomGraphBase() = default;  
+  virtual ~CustomGraphBase() = default;
 };
 
 // ----------------------------------------------------------------------------
@@ -42,7 +41,7 @@ class CustomGraphBase {
 /**
 @class Graph
 
-@brief class to create a graph object 
+@brief class to create a graph object
 
 A graph is the ultimate storage for a task dependency graph and is the main
 gateway to interact with an executor.
@@ -63,7 +62,7 @@ class Graph {
   friend class Executor;
 
   public:
-    
+
     /**
     @brief constructs a graph object
     */
@@ -83,7 +82,7 @@ class Graph {
     @brief destructs the graph object
     */
     ~Graph();
-    
+
     /**
     @brief disabled copy assignment operator
     */
@@ -93,30 +92,35 @@ class Graph {
     @brief assigns a graph using move semantics
     */
     Graph& operator = (Graph&&);
-    
+
     /**
     @brief queries if the graph is empty
     */
     bool empty() const;
-    
+
     /**
     @brief queries the number of nodes in the graph
     */
     size_t size() const;
 
+    /**
+    @brief clears the graph
+    */
+    void clear();
+
   private:
 
     std::vector<Node*> _nodes;
-    
+
     void _clear();
     void _clear_detached();
     void _merge(Graph&&);
     void _erase(Node*);
-    
+
     template <typename ...ArgsT>
-    Node* _emplace_back(ArgsT&&... args); 
-    
-    Node* _emplace_back(); 
+    Node* _emplace_back(ArgsT&&... args);
+
+    Node* _emplace_back();
 };
 
 // ----------------------------------------------------------------------------
@@ -152,7 +156,7 @@ class Runtime {
   /**
   @brief obtains the running executor
 
-  The running executor of a runtime task is the executor that runs 
+  The running executor of a runtime task is the executor that runs
   the parent taskflow of that runtime task.
 
   @code{.cpp}
@@ -165,14 +169,16 @@ class Runtime {
   @endcode
   */
   Executor& executor();
-  
+
   /**
   @brief schedules an active task immediately to the worker's queue
 
+  @param task the given active task to schedule immediately
+
   This member function immediately schedules an active task to the
   task queue of the associated worker in the runtime task.
-  An active task is a task in a running taskflow. 
-  The task may or may not be running, and scheduling that task 
+  An active task is a task in a running taskflow.
+  The task may or may not be running, and scheduling that task
   will immediately put the task into the task queue of the worker
   that is running the runtime task.
   Consider the following example:
@@ -182,7 +188,7 @@ class Runtime {
   std::tie(A, B, C, D) = taskflow.emplace(
     [] () { return 0; },
     [&C] (tf::Runtime& rt) {  // C must be captured by reference
-      std::cout << "B\n"; 
+      std::cout << "B\n";
       rt.schedule(C);
     },
     [] () { std::cout << "C\n"; },
@@ -192,23 +198,44 @@ class Runtime {
   executor.run(taskflow).wait();
   @endcode
 
-  The executor will first run the condition task @c A which returns @c 0 
-  to inform the scheduler to go to the runtime task @c B. 
+  The executor will first run the condition task @c A which returns @c 0
+  to inform the scheduler to go to the runtime task @c B.
   During the execution of @c B, it directly schedules task @c C without
   going through the normal taskflow graph scheduling process.
-  At this moment, task @c C is active because its parent taskflow is running. 
+  At this moment, task @c C is active because its parent taskflow is running.
   When the taskflow finishes, we will see both @c B and @c C in the output.
   */
-  void schedule(Task);
-  
+  void schedule(Task task);
+
   /**
-  @brief runs a task callable synchronously
+  @brief runs the given target and waits until it completes
+  
+  A target can be 
+  (1) a callable to spawn a subflow or
+  (2) a composable target with `tf::Graph& T::graph()` defined
+
+  @code{.cpp}
+  // complete a subflow synchronously
+  taskflow.emplace([](tf::Runtime& rt){
+    rt.run_and_wait([](tf::Subflow& sf){
+      tf::Task A = sf.emplace([](){});
+      tf::Task B = sf.emplace([](){});
+    }); 
+  });
+  
+  // complete a custom graph synchronously
+  tf::Taskflow taskflow;
+  taskflow.emplace([](){});
+  taskflow.emplace([&](tf::Runtime& rt){
+    rt.run_and_wait(taskflow);
+  });
+  @endcode
   */
-  template <typename C>
-  void run(C&&);
+  template <typename T>
+  void run_and_wait(T&& target);
 
   private:
-  
+
   explicit Runtime(Executor&, Worker&, Node*);
 
   Executor& _executor;
@@ -217,7 +244,7 @@ class Runtime {
 };
 
 // constructor
-inline Runtime::Runtime(Executor& e, Worker& w, Node* p) : 
+inline Runtime::Runtime(Executor& e, Worker& w, Node* p) :
   _executor{e},
   _worker  {w},
   _parent  {p}{
@@ -236,7 +263,7 @@ inline Executor& Runtime::executor() {
 @private
 */
 class Node {
-  
+
   friend class Graph;
   friend class Task;
   friend class TaskView;
@@ -254,11 +281,11 @@ class Node {
   constexpr static int ACQUIRED    = 4;
   constexpr static int READY       = 8;
   constexpr static int DEFERRED    = 16;
-  
+
   // static work handle
   struct Static {
 
-    template <typename C> 
+    template <typename C>
     Static(C&&);
 
     std::function<void()> work;
@@ -269,24 +296,24 @@ class Node {
 
     template <typename C>
     Runtime(C&&);
-     
+
     std::function<void(tf::Runtime&)> work;
   };
 
   // dynamic work handle
   struct Dynamic {
 
-    template <typename C> 
+    template <typename C>
     Dynamic(C&&);
 
     std::function<void(Subflow&)> work;
     Graph subgraph;
   };
-  
+
   // condition work handle
   struct Condition {
 
-    template <typename C> 
+    template <typename C>
     Condition(C&&);
 
     std::function<int()> work;
@@ -320,38 +347,38 @@ class Node {
 
     std::shared_ptr<AsyncTopology> topology;
   };
-  
+
   // Silent async work
   struct SilentAsync {
-    
+
     template <typename C>
     SilentAsync(C&&);
 
     std::function<void()> work;
   };
-  
+
   // cudaFlow work handle
   struct cudaFlow {
-    
-    template <typename C, typename G> 
+
+    template <typename C, typename G>
     cudaFlow(C&& c, G&& g);
 
     std::function<void(Executor&, Node*)> work;
 
     std::unique_ptr<CustomGraphBase> graph;
   };
-  
+
   // syclFlow work handle
   struct syclFlow {
-    
-    template <typename C, typename G> 
+
+    template <typename C, typename G>
     syclFlow(C&& c, G&& g);
 
     std::function<void(Executor&, Node*)> work;
 
     std::unique_ptr<CustomGraphBase> graph;
   };
-    
+
   using handle_t = std::variant<
     std::monostate,  // placeholder
     Static,          // static tasking
@@ -365,25 +392,25 @@ class Node {
     syclFlow,        // syclFlow
     Runtime          // runtime tasking
   >;
-    
-  struct Semaphores {  
+
+  struct Semaphores {
     SmallVector<Semaphore*> to_acquire;
     SmallVector<Semaphore*> to_release;
   };
 
   public:
-  
+
   // variant index
   constexpr static auto PLACEHOLDER     = get_index_v<std::monostate, handle_t>;
   constexpr static auto STATIC          = get_index_v<Static, handle_t>;
   constexpr static auto DYNAMIC         = get_index_v<Dynamic, handle_t>;
-  constexpr static auto CONDITION       = get_index_v<Condition, handle_t>; 
-  constexpr static auto MULTI_CONDITION = get_index_v<MultiCondition, handle_t>; 
-  constexpr static auto MODULE          = get_index_v<Module, handle_t>; 
-  constexpr static auto ASYNC           = get_index_v<Async, handle_t>; 
-  constexpr static auto SILENT_ASYNC    = get_index_v<SilentAsync, handle_t>; 
-  constexpr static auto CUDAFLOW        = get_index_v<cudaFlow, handle_t>; 
-  constexpr static auto SYCLFLOW        = get_index_v<syclFlow, handle_t>; 
+  constexpr static auto CONDITION       = get_index_v<Condition, handle_t>;
+  constexpr static auto MULTI_CONDITION = get_index_v<MultiCondition, handle_t>;
+  constexpr static auto MODULE          = get_index_v<Module, handle_t>;
+  constexpr static auto ASYNC           = get_index_v<Async, handle_t>;
+  constexpr static auto SILENT_ASYNC    = get_index_v<SilentAsync, handle_t>;
+  constexpr static auto CUDAFLOW        = get_index_v<cudaFlow, handle_t>;
+  constexpr static auto SYCLFLOW        = get_index_v<syclFlow, handle_t>;
   constexpr static auto RUNTIME         = get_index_v<Runtime, handle_t>;
 
   template <typename... Args>
@@ -410,14 +437,14 @@ class Node {
   SmallVector<Node*> _dependents;
 
   Topology* _topology {nullptr};
-  
+
   Node* _parent {nullptr};
 
   std::atomic<int> _state {0};
   std::atomic<size_t> _join_counter {0};
-  
+
   std::unique_ptr<Semaphores> _semaphores;
-  
+
   void _precede(Node*);
   void _set_up_join_counter();
 
@@ -440,36 +467,36 @@ inline ObjectPool<Node> node_pool;
 // ----------------------------------------------------------------------------
 // Definition for Node::Static
 // ----------------------------------------------------------------------------
-    
+
 // Constructor
-template <typename C> 
+template <typename C>
 Node::Static::Static(C&& c) : work {std::forward<C>(c)} {
 }
 
 // ----------------------------------------------------------------------------
 // Definition for Node::Dynamic
 // ----------------------------------------------------------------------------
-    
+
 // Constructor
-template <typename C> 
+template <typename C>
 Node::Dynamic::Dynamic(C&& c) : work {std::forward<C>(c)} {
 }
 
 // ----------------------------------------------------------------------------
 // Definition for Node::Condition
 // ----------------------------------------------------------------------------
-    
+
 // Constructor
-template <typename C> 
+template <typename C>
 Node::Condition::Condition(C&& c) : work {std::forward<C>(c)} {
 }
 
 // ----------------------------------------------------------------------------
 // Definition for Node::MultiCondition
 // ----------------------------------------------------------------------------
-    
+
 // Constructor
-template <typename C> 
+template <typename C>
 Node::MultiCondition::MultiCondition(C&& c) : work {std::forward<C>(c)} {
 }
 
@@ -492,7 +519,7 @@ Node::syclFlow::syclFlow(C&& c, G&& g) :
   work  {std::forward<C>(c)},
   graph {std::forward<G>(g)} {
 }
-    
+
 // ----------------------------------------------------------------------------
 // Definition for Node::Module
 // ----------------------------------------------------------------------------
@@ -505,10 +532,10 @@ inline Node::Module::Module(T& obj) : graph{ obj.graph() } {
 // ----------------------------------------------------------------------------
 // Definition for Node::Async
 // ----------------------------------------------------------------------------
-    
+
 // Constructor
 template <typename C>
-Node::Async::Async(C&& c, std::shared_ptr<AsyncTopology>tpg) : 
+Node::Async::Async(C&& c, std::shared_ptr<AsyncTopology>tpg) :
   work     {std::forward<C>(c)},
   topology {std::move(tpg)} {
 }
@@ -540,16 +567,16 @@ Node::Runtime::Runtime(C&& c) :
 // Constructor
 template <typename... Args>
 Node::Node(Args&&... args): _handle{std::forward<Args>(args)...} {
-} 
+}
 
 // Destructor
 inline Node::~Node() {
   // this is to avoid stack overflow
 
   if(_handle.index() == DYNAMIC) {
-    // using std::get_if instead of std::get makes this compatible 
+    // using std::get_if instead of std::get makes this compatible
     // with older macOS versions
-    // the result of std::get_if is guaranteed to be non-null 
+    // the result of std::get_if is guaranteed to be non-null
     // due to the index check above
     auto& subgraph = std::get_if<Dynamic>(&_handle)->subgraph;
     std::vector<Node*> nodes;
@@ -574,7 +601,7 @@ inline Node::~Node() {
 
       ++i;
     }
-      
+
     //auto& np = Graph::_node_pool();
     for(i=0; i<nodes.size(); ++i) {
       node_pool.recycle(nodes[i]);
@@ -695,14 +722,14 @@ inline SmallVector<Node*> Node::_release_all() {
 // ----------------------------------------------------------------------------
 // Graph definition
 // ----------------------------------------------------------------------------
-    
+
 // Destructor
 inline Graph::~Graph() {
   _clear();
 }
 
 // Move constructor
-inline Graph::Graph(Graph&& other) : 
+inline Graph::Graph(Graph&& other) :
   _nodes {std::move(other._nodes)} {
 }
 
@@ -713,6 +740,11 @@ inline Graph& Graph::operator = (Graph&& other) {
   return *this;
 }
 
+// Procedure: clear
+inline void Graph::clear() {
+  _clear();
+}
+
 // Procedure: clear
 inline void Graph::_clear() {
   for(auto node : _nodes) {
@@ -727,7 +759,7 @@ inline void Graph::_clear_detached() {
   auto mid = std::partition(_nodes.begin(), _nodes.end(), [] (Node* node) {
     return !(node->_state.load(std::memory_order_relaxed) & Node::DETACHED);
   });
-  
+
   for(auto itr = mid; itr != _nodes.end(); ++itr) {
     node_pool.recycle(*itr);
   }
@@ -759,7 +791,7 @@ inline size_t Graph::size() const {
 inline bool Graph::empty() const {
   return _nodes.empty();
 }
-    
+
 // Function: emplace_back
 template <typename ...ArgsT>
 Node* Graph::_emplace_back(ArgsT&&... args) {
