@@ -30,15 +30,15 @@ struct Segment {
   observer_stamp_t beg;
   observer_stamp_t end;
 
-  template <typename Archiver>
-  auto save(Archiver& ar) const {
-    return ar(name, type, beg, end);
-  }
+  //template <typename Archiver>
+  //auto save(Archiver& ar) const {
+  //  return ar(name, type, beg, end);
+  //}
 
-  template <typename Archiver>
-  auto load(Archiver& ar) {
-    return ar(name, type, beg, end);
-  }
+  //template <typename Archiver>
+  //auto load(Archiver& ar) {
+  //  return ar(name, type, beg, end);
+  //}
 
   Segment() = default;
 
@@ -70,15 +70,15 @@ struct Timeline {
   Timeline& operator = (const Timeline& rhs) = delete;
   Timeline& operator = (Timeline&& rhs) = default;
 
-  template <typename Archiver>
-  auto save(Archiver& ar) const {
-    return ar(uid, origin, segments);
-  }
+  //template <typename Archiver>
+  //auto save(Archiver& ar) const {
+  //  return ar(uid, origin, segments);
+  //}
 
-  template <typename Archiver>
-  auto load(Archiver& ar) {
-    return ar(uid, origin, segments);
-  }
+  //template <typename Archiver>
+  //auto load(Archiver& ar) {
+  //  return ar(uid, origin, segments);
+  //}
 };  
 
 /**
@@ -96,15 +96,15 @@ struct ProfileData {
   ProfileData& operator = (const ProfileData& rhs) = delete;
   ProfileData& operator = (ProfileData&&) = default;
   
-  template <typename Archiver>
-  auto save(Archiver& ar) const {
-    return ar(timelines);
-  }
+  //template <typename Archiver>
+  //auto save(Archiver& ar) const {
+  //  return ar(timelines);
+  //}
 
-  template <typename Archiver>
-  auto load(Archiver& ar) {
-    return ar(timelines);
-  }
+  //template <typename Archiver>
+  //auto load(Archiver& ar) {
+  //  return ar(timelines);
+  //}
 };
 
 // ----------------------------------------------------------------------------
@@ -116,7 +116,7 @@ struct ProfileData {
 
 @brief class to derive an executor observer 
 
-The tf::ObserverInterface class let users define custom methods to monitor 
+The tf::ObserverInterface class allows users to define custom methods to monitor 
 the behaviors of an executor. This is particularly useful when you want to 
 inspect the performance of an executor and visualize when each thread 
 participates in the execution of a task.
@@ -168,8 +168,6 @@ executor.run(taskflow).wait();
 */
 class ObserverInterface {
 
-  friend class Executor;
-  
   public:
 
   /**
@@ -185,17 +183,17 @@ class ObserverInterface {
   
   /**
   @brief method to call before a worker thread executes a closure 
-  @param w an immutable view of this worker thread 
+  @param wv an immutable view of this worker thread 
   @param task_view a constant wrapper object to the task 
   */
-  virtual void on_entry(WorkerView w, TaskView task_view) = 0;
+  virtual void on_entry(WorkerView wv, TaskView task_view) = 0;
   
   /**
   @brief method to call after a worker thread executed a closure
-  @param w an immutable view of this worker thread
+  @param wv an immutable view of this worker thread
   @param task_view a constant wrapper object to the task
   */
-  virtual void on_exit(WorkerView w, TaskView task_view) = 0;
+  virtual void on_exit(WorkerView wv, TaskView task_view) = 0;
 };
 
 // ----------------------------------------------------------------------------
@@ -675,25 +673,32 @@ inline void TFProfManager::dump(std::ostream& os) const {
 inline TFProfManager::~TFProfManager() {
   std::ofstream ofs(_fpath);
   if(ofs) {
-    // .tfp
-    if(_fpath.rfind(".tfp") != std::string::npos) {
-      ProfileData data;
-      data.timelines.reserve(_observers.size());
-      for(size_t i=0; i<_observers.size(); ++i) {
-        data.timelines.push_back(std::move(_observers[i]->_timeline));
-      }
-      Serializer<std::ofstream> serializer(ofs); 
-      serializer(data);
-    }
-    // .json
-    else {
-      ofs << "[\n";
-      for(size_t i=0; i<_observers.size(); ++i) {
-        if(i) ofs << ',';
-        _observers[i]->dump(ofs);
-      }
-      ofs << "]\n";
+    //// .tfp
+    //if(_fpath.rfind(".tfp") != std::string::npos) {
+    //  ProfileData data;
+    //  data.timelines.reserve(_observers.size());
+    //  for(size_t i=0; i<_observers.size(); ++i) {
+    //    data.timelines.push_back(std::move(_observers[i]->_timeline));
+    //  }
+    //  Serializer<std::ofstream> serializer(ofs); 
+    //  serializer(data);
+    //}
+    //// .json
+    //else {
+    //  ofs << "[\n";
+    //  for(size_t i=0; i<_observers.size(); ++i) {
+    //    if(i) ofs << ',';
+    //    _observers[i]->dump(ofs);
+    //  }
+    //  ofs << "]\n";
+    //}
+
+    ofs << "[\n";
+    for(size_t i=0; i<_observers.size(); ++i) {
+      if(i) ofs << ',';
+      _observers[i]->dump(ofs);
     }
+    ofs << "]\n";
   }
 }
     
