@@ -2,10 +2,9 @@
 
 #include "cuda_memory.hpp"
 #include "cuda_stream.hpp"
+#include "cuda_meta.hpp"
 
-#include "../utility/object_pool.hpp"
 #include "../utility/traits.hpp"
-#include "../utility/passive_vector.hpp"
 
 namespace tf {
 
@@ -16,7 +15,7 @@ namespace tf {
 /**
 @brief gets the memcpy node parameter of a copy task
 */
-template <typename T, 
+template <typename T,
   std::enable_if_t<!std::is_same_v<T, void>, void>* = nullptr
 >
 cudaMemcpy3DParms cuda_get_copy_parms(T* tgt, const T* src, size_t num) {
@@ -87,7 +86,7 @@ template <typename T, std::enable_if_t<
   is_pod_v<T> && (sizeof(T)==1 || sizeof(T)==2 || sizeof(T)==4), void>* = nullptr
 >
 cudaMemsetParams cuda_get_fill_parms(T* dst, T value, size_t count) {
-  
+
   cudaMemsetParams p;
   p.dst = dst;
 
@@ -129,10 +128,10 @@ cudaMemsetParams cuda_get_zero_parms(T* dst, size_t count) {
 inline size_t cuda_get_graph_num_root_nodes(cudaGraph_t graph) {
   size_t num_nodes;
   TF_CHECK_CUDA(
-    cudaGraphGetRootNodes(graph, nullptr, &num_nodes), 
+    cudaGraphGetRootNodes(graph, nullptr, &num_nodes),
     "failed to get native graph root nodes"
   );
-  return num_nodes; 
+  return num_nodes;
 }
 
 /**
@@ -141,10 +140,10 @@ inline size_t cuda_get_graph_num_root_nodes(cudaGraph_t graph) {
 inline size_t cuda_get_graph_num_nodes(cudaGraph_t graph) {
   size_t num_nodes;
   TF_CHECK_CUDA(
-    cudaGraphGetNodes(graph, nullptr, &num_nodes), 
+    cudaGraphGetNodes(graph, nullptr, &num_nodes),
     "failed to get native graph nodes"
   );
-  return num_nodes; 
+  return num_nodes;
 }
 
 /**
@@ -153,7 +152,7 @@ inline size_t cuda_get_graph_num_nodes(cudaGraph_t graph) {
 inline size_t cuda_get_graph_num_edges(cudaGraph_t graph) {
   size_t num_edges;
   TF_CHECK_CUDA(
-    cudaGraphGetEdges(graph, nullptr, nullptr, &num_edges), 
+    cudaGraphGetEdges(graph, nullptr, nullptr, &num_edges),
     "failed to get native graph edges"
   );
   return num_edges;
@@ -250,7 +249,7 @@ inline const char* cuda_graph_node_type_to_string(cudaGraphNodeType type) {
 */
 template <typename T>
 void cuda_dump_graph(T& os, cudaGraph_t graph) {
-  
+
   os << "digraph cudaGraph {\n";
 
   std::stack<std::tuple<cudaGraph_t, cudaGraphNode_t, int>> stack;
@@ -277,7 +276,7 @@ void cuda_dump_graph(T& os, cudaGraph_t graph) {
     for(auto& [from, to] : edges) {
       os << 'p' << from << " -> " << 'p' << to << ";\n";
     }
-  
+
     for(auto& node : nodes) {
       auto type = cuda_get_graph_node_type(node);
       if(type == cudaGraphNodeTypeGraph) {
@@ -292,8 +291,8 @@ void cuda_dump_graph(T& os, cudaGraph_t graph) {
            << "\"];\n";
       }
       else {
-        os << 'p' << node << "[label=\"" 
-           << cuda_graph_node_type_to_string(type) 
+        os << 'p' << node << "[label=\""
+           << cuda_graph_node_type_to_string(type)
            << "\"];\n";
       }
     }
@@ -310,7 +309,7 @@ void cuda_dump_graph(T& os, cudaGraph_t graph) {
         }
       }
     }
-    
+
     // set the previous level
     pl = l;
   }
@@ -320,6 +319,236 @@ void cuda_dump_graph(T& os, cudaGraph_t graph) {
   }
 }
 
+// ----------------------------------------------------------------------------
+// cudaGraphNative
+// ----------------------------------------------------------------------------
+
+/**
+@class cudaGraphNative
+
+@brief class to create an RAII-styled wrapper over a CUDA executable graph
+
+A cudaGraphNative object is an RAII-styled wrapper over 
+a native CUDA executable graph (@c cudaGraphNative_t).
+A cudaGraphNative object is move-only.
+*/
+class cudaGraphNative {
+
+  struct cudaGraphNativeCreator {
+    cudaGraph_t operator () () const { 
+      cudaGraph_t g;
+      TF_CHECK_CUDA(cudaGraphCreate(&g, 0), "failed to create a CUDA native graph");
+      return g; 
+    }
+  };
+  
+  struct cudaGraphNativeDeleter {
+    void operator () (cudaGraph_t g) const {
+      if(g) {
+        cudaGraphDestroy(g);
+      }
+    }
+  };
+
+  public:
+
+    /**
+    @brief constructs an RAII-styled object from the given CUDA exec
+
+    Constructs a cudaGraphNative object which owns @c exec.
+    */
+    explicit cudaGraphNative(cudaGraph_t native) : _native(native) {
+    }
+    
+    /**
+    @brief constructs an RAII-styled object for a new CUDA exec
+
+    Equivalently calling @c cudaGraphNativeCreate to create a exec.
+    */
+    cudaGraphNative() : _native{ cudaGraphNativeCreator{}() } {
+    }
+    
+    /**
+    @brief disabled copy constructor
+    */
+    cudaGraphNative(const cudaGraphNative&) = delete;
+    
+    /**
+    @brief move constructor
+    */
+    cudaGraphNative(cudaGraphNative&& rhs) : _native{rhs._native} {
+      rhs._native = nullptr;
+    }
+
+    /**
+    @brief destructs the CUDA exec
+    */
+    ~cudaGraphNative() {
+      cudaGraphNativeDeleter {} (_native);
+    }
+    
+    /**
+    @brief disabled copy assignment
+    */
+    cudaGraphNative& operator = (const cudaGraphNative&) = delete;
+
+    /**
+    @brief move assignment
+    */
+    cudaGraphNative& operator = (cudaGraphNative&& rhs) {
+      cudaGraphNativeDeleter {} (_native);
+      _native = rhs._native;
+      rhs._native = nullptr;
+      return *this;
+    }
+    
+    /**
+    @brief implicit conversion to the native CUDA exec (cudaGraphNative_t)
+
+    Returns the underlying exec of type @c cudaGraphNative_t.
+    */
+    operator cudaGraph_t () const {
+      return _native;
+    }
+    
+  private:
+
+    cudaGraph_t _native {nullptr};
+};
+
+// ----------------------------------------------------------------------------
+// cudaGraphExec
+// ----------------------------------------------------------------------------
+
+/**
+@class cudaGraphExec
+
+@brief class to create an RAII-styled wrapper over a CUDA executable graph
+
+A cudaGraphExec object is an RAII-styled wrapper over 
+a native CUDA executable graph (@c cudaGraphExec_t).
+A cudaGraphExec object is move-only.
+*/
+class cudaGraphExec {
+
+  struct cudaGraphExecCreator {
+    cudaGraphExec_t operator () () const { return nullptr; }
+  };
+  
+  struct cudaGraphExecDeleter {
+    void operator () (cudaGraphExec_t executable) const {
+      if(executable) {
+        cudaGraphExecDestroy(executable);
+      }
+    }
+  };
+
+  public:
+
+    /**
+    @brief constructs an RAII-styled object from the given CUDA exec
+
+    Constructs a cudaGraphExec object which owns @c exec.
+    */
+    explicit cudaGraphExec(cudaGraphExec_t exec) : _exec(exec) {
+    }
+    
+    /**
+    @brief constructs an RAII-styled object for a new CUDA exec
+
+    Equivalently calling @c cudaGraphExecCreate to create a exec.
+    */
+    cudaGraphExec() : _exec{ cudaGraphExecCreator{}() } {
+    }
+    
+    /**
+    @brief disabled copy constructor
+    */
+    cudaGraphExec(const cudaGraphExec&) = delete;
+    
+    /**
+    @brief move constructor
+    */
+    cudaGraphExec(cudaGraphExec&& rhs) : _exec{rhs._exec} {
+      rhs._exec = nullptr;
+    }
+
+    /**
+    @brief destructs the CUDA exec
+    */
+    ~cudaGraphExec() {
+      cudaGraphExecDeleter {} (_exec);
+    }
+    
+    /**
+    @brief disabled copy assignment
+    */
+    cudaGraphExec& operator = (const cudaGraphExec&) = delete;
+
+    /**
+    @brief move assignment
+    */
+    cudaGraphExec& operator = (cudaGraphExec&& rhs) {
+      cudaGraphExecDeleter {} (_exec);
+      _exec = rhs._exec;
+      rhs._exec = nullptr;
+      return *this;
+    }
+    
+    /**
+    @brief replaces the managed executable graph with the given one
+
+    Destructs the managed exec and resets it to the given exec.
+    */
+    void clear() {
+      cudaGraphExecDeleter {} (_exec);
+      _exec = nullptr;
+    }
+    
+    /**
+    @brief instantiates the exexutable from the given CUDA graph
+    */
+    void instantiate(cudaGraph_t graph) {
+      cudaGraphExecDeleter {} (_exec);
+      TF_CHECK_CUDA(
+        cudaGraphInstantiate(&_exec, graph, nullptr, nullptr, 0),
+        "failed to create an executable graph"
+      );
+    }
+    
+    /**
+    @brief updates the exexutable from the given CUDA graph
+    */
+    cudaGraphExecUpdateResult update(cudaGraph_t graph) {
+      cudaGraphNode_t error_node;
+      cudaGraphExecUpdateResult error_result;
+      cudaGraphExecUpdate(_exec, graph, &error_node, &error_result);
+      return error_result;
+    }
+    
+    /**
+    @brief launchs the executable graph via the given stream
+    */
+    void launch(cudaStream_t stream) {
+      TF_CHECK_CUDA(
+        cudaGraphLaunch(_exec, stream), "failed to launch a CUDA executable graph"
+      );
+    }
+  
+    /**
+    @brief implicit conversion to the native CUDA exec (cudaGraphExec_t)
+
+    Returns the underlying exec of type @c cudaGraphExec_t.
+    */
+    operator cudaGraphExec_t () const {
+      return _exec;
+    }
+    
+  private:
+
+    cudaGraphExec_t _exec {nullptr};
+};
+
 // ----------------------------------------------------------------------------
 // cudaGraph class
 // ----------------------------------------------------------------------------
@@ -332,58 +561,68 @@ class cudaGraph : public CustomGraphBase {
   friend class cudaFlowCapturerBase;
   friend class cudaFlowCapturer;
   friend class cudaFlow;
-  
+  friend class cudaCapturingBase;
+  friend class cudaSequentialCapturing;
+  friend class cudaLinearCapturing;
+  friend class cudaRoundRobinCapturing;
   friend class Taskflow;
   friend class Executor;
 
+  constexpr static int OFFLOADED = 0x01;
+  constexpr static int CHANGED   = 0x02;
+  constexpr static int UPDATED   = 0x04;
+
   public:
-    
+
     cudaGraph() = default;
     ~cudaGraph();
 
     cudaGraph(const cudaGraph&) = delete;
     cudaGraph(cudaGraph&&);
-    
+
     cudaGraph& operator = (const cudaGraph&) = delete;
     cudaGraph& operator = (cudaGraph&&);
 
     template <typename... ArgsT>
     cudaNode* emplace_back(ArgsT&&...);
 
-    void clear();
-
     bool empty() const;
 
+    void clear();
     void dump(std::ostream&, const void*, const std::string&) const override final;
 
   private:
 
+    int _state{CHANGED};
+
     cudaGraph_t _native_handle {nullptr};
-    
-    // TODO: nvcc complains deleter of unique_ptr
-    //std::vector<std::unique_ptr<cudaNode>> _nodes;
-    std::vector<cudaNode*> _nodes;
-    std::vector<cudaNode*> _toposort();
+
+    std::vector<std::unique_ptr<cudaNode>> _nodes;
+    //std::vector<cudaNode*> _nodes;
 };
 
 // ----------------------------------------------------------------------------
 // cudaNode class
 // ----------------------------------------------------------------------------
 
-// class: cudaNode
-// each create_native_node is wrapped in a function to call at runtime 
-// in order to work with gpu context
+/**
+@private
+@class: cudaNode
+*/
 class cudaNode {
-  
+
   friend class cudaGraph;
   friend class cudaTask;
   friend class cudaFlow;
   friend class cudaFlowCapturer;
   friend class cudaFlowCapturerBase;
-
+  friend class cudaCapturingBase;
+  friend class cudaSequentialCapturing;
+  friend class cudaLinearCapturing;
+  friend class cudaRoundRobinCapturing;
   friend class Taskflow;
   friend class Executor;
-  
+
   // Empty handle
   struct Empty {
   };
@@ -395,7 +634,7 @@ class cudaNode {
     Host(C&&);
 
     std::function<void()> func;
-    
+
     static void callback(void*);
   };
 
@@ -406,13 +645,13 @@ class cudaNode {
   // Memcpy handle
   struct Memcpy {
   };
-  
+
   // Kernel handle
   struct Kernel {
-    
+
     template <typename F>
     Kernel(F&& f);
-    
+
     void* func {nullptr};
   };
 
@@ -423,60 +662,58 @@ class cudaNode {
 
   // Capture
   struct Capture {
-    
+
     template <typename C>
     Capture(C&&);
-    
+
     std::function<void(cudaStream_t)> work;
+
+    cudaEvent_t event;
+    size_t level;
+    size_t lid;
+    size_t idx;
   };
 
   using handle_t = std::variant<
-    Empty, 
+    Empty,
     Host,
-    Memset, 
-    Memcpy, 
+    Memset,
+    Memcpy,
     Kernel,
     Subflow,
     Capture
   >;
 
-  constexpr static auto STATE_VISITED = 0x1;
-
   public:
-  
+
   // variant index
-  constexpr static auto CUDA_EMPTY_TASK   = get_index_v<Empty, handle_t>;
-  constexpr static auto CUDA_HOST_TASK    = get_index_v<Host, handle_t>;
-  constexpr static auto CUDA_MEMSET_TASK  = get_index_v<Memset, handle_t>;
-  constexpr static auto CUDA_MEMCPY_TASK  = get_index_v<Memcpy, handle_t>; 
-  constexpr static auto CUDA_KERNEL_TASK  = get_index_v<Kernel, handle_t>;
-  constexpr static auto CUDA_SUBFLOW_TASK = get_index_v<Subflow, handle_t>;
-  constexpr static auto CUDA_CAPTURE_TASK = get_index_v<Capture, handle_t>;
+  constexpr static auto EMPTY   = get_index_v<Empty, handle_t>;
+  constexpr static auto HOST    = get_index_v<Host, handle_t>;
+  constexpr static auto MEMSET  = get_index_v<Memset, handle_t>;
+  constexpr static auto MEMCPY  = get_index_v<Memcpy, handle_t>;
+  constexpr static auto KERNEL  = get_index_v<Kernel, handle_t>;
+  constexpr static auto SUBFLOW = get_index_v<Subflow, handle_t>;
+  constexpr static auto CAPTURE = get_index_v<Capture, handle_t>;
 
     cudaNode() = delete;
-    
+
     template <typename... ArgsT>
     cudaNode(cudaGraph&, ArgsT&&...);
 
   private:
-    
-    int _state;
 
     cudaGraph& _graph;
 
     std::string _name;
-    
+
     handle_t _handle;
 
     cudaGraphNode_t _native_handle {nullptr};
 
-    std::vector<cudaNode*> _successors;
+    SmallVector<cudaNode*> _successors;
+    SmallVector<cudaNode*> _dependents;
 
     void _precede(cudaNode*);
-    void _set_state(int);
-    void _unset_state(int);
-    void _clear_state();
-    bool _has_state(int) const;
 };
 
 // ----------------------------------------------------------------------------
@@ -488,26 +725,26 @@ template <typename C>
 cudaNode::Host::Host(C&& c) : func {std::forward<C>(c)} {
 }
 
-// Host callback    
-inline void cudaNode::Host::callback(void* data) { 
-  static_cast<Host*>(data)->func(); 
+// Host callback
+inline void cudaNode::Host::callback(void* data) {
+  static_cast<Host*>(data)->func();
 };
 
 // Kernel handle constructor
 template <typename F>
-cudaNode::Kernel::Kernel(F&& f) : 
+cudaNode::Kernel::Kernel(F&& f) :
   func {std::forward<F>(f)} {
 }
 
 // Capture handle constructor
 template <typename C>
-cudaNode::Capture::Capture(C&& work) : 
+cudaNode::Capture::Capture(C&& work) :
   work {std::forward<C>(work)} {
 }
 
 // Constructor
 template <typename... ArgsT>
-cudaNode::cudaNode(cudaGraph& graph, ArgsT&&... args) : 
+cudaNode::cudaNode(cudaGraph& graph, ArgsT&&... args) :
   _graph {graph},
   _handle {std::forward<ArgsT>(args)...} {
 }
@@ -515,38 +752,41 @@ cudaNode::cudaNode(cudaGraph& graph, ArgsT&&... args) :
 // Procedure: _precede
 inline void cudaNode::_precede(cudaNode* v) {
 
+  _graph._state |= cudaGraph::CHANGED;
+
   _successors.push_back(v);
+  v->_dependents.push_back(this);
 
   // capture node doesn't have the native graph yet
-  if(_handle.index() != CUDA_CAPTURE_TASK) {
+  if(_handle.index() != cudaNode::CAPTURE) {
     TF_CHECK_CUDA(
-      ::cudaGraphAddDependencies(
+      cudaGraphAddDependencies(
         _graph._native_handle, &_native_handle, &v->_native_handle, 1
       ),
       "failed to add a preceding link ", this, "->", v
     );
   }
 }
-    
-// Procedure: _set_state
-inline void cudaNode::_set_state(int flag) { 
-  _state |= flag; 
-}
 
-// Procedure: _unset_state
-inline void cudaNode::_unset_state(int flag) { 
-  _state &= ~flag; 
-}
-
-// Procedure: _clear_state
-inline void cudaNode::_clear_state() { 
-  _state = 0; 
-}
-
-// Function: _has_state
-inline bool cudaNode::_has_state(int flag) const {
-  return _state & flag;
-}
+//// Procedure: _set_state
+//inline void cudaNode::_set_state(int flag) {
+//  _state |= flag;
+//}
+//
+//// Procedure: _unset_state
+//inline void cudaNode::_unset_state(int flag) {
+//  _state &= ~flag;
+//}
+//
+//// Procedure: _clear_state
+//inline void cudaNode::_clear_state() {
+//  _state = 0;
+//}
+//
+//// Function: _has_state
+//inline bool cudaNode::_has_state(int flag) const {
+//  return _state & flag;
+//}
 
 // ----------------------------------------------------------------------------
 // cudaGraph definitions
@@ -554,7 +794,7 @@ inline bool cudaNode::_has_state(int flag) const {
 
 // Destructor
 inline cudaGraph::~cudaGraph() {
-  clear();
+  //clear();
   assert(_native_handle == nullptr);
 }
 
@@ -562,7 +802,7 @@ inline cudaGraph::~cudaGraph() {
 inline cudaGraph::cudaGraph(cudaGraph&& g) :
   _native_handle {g._native_handle},
   _nodes         {std::move(g._nodes)} {
-  
+
   g._native_handle = nullptr;
 
   assert(g._nodes.empty());
@@ -571,8 +811,8 @@ inline cudaGraph::cudaGraph(cudaGraph&& g) :
 // Move assignment
 inline cudaGraph& cudaGraph::operator = (cudaGraph&& rhs) {
 
-  clear();
-  
+  //clear();
+
   // lhs
   _native_handle = rhs._native_handle;
   _nodes = std::move(rhs._nodes);
@@ -582,7 +822,7 @@ inline cudaGraph& cudaGraph::operator = (cudaGraph&& rhs) {
   // rhs
   rhs._native_handle = nullptr;
 
-  return *this; 
+  return *this;
 }
 
 // Function: empty
@@ -592,71 +832,34 @@ inline bool cudaGraph::empty() const {
 
 // Procedure: clear
 inline void cudaGraph::clear() {
-  for(auto n : _nodes) {
-    delete n;
-  }
+  //for(auto n : _nodes) {
+  //  delete n;
+  //}
+  _state = cudaGraph::CHANGED;
   _nodes.clear();
 }
 
 // Function: emplace_back
 template <typename... ArgsT>
 cudaNode* cudaGraph::emplace_back(ArgsT&&... args) {
-  //auto node = std::make_unique<cudaNode>(std::forward<ArgsT>(args)...);
-  //_nodes.emplace_back(std::move(node));
-  //return _nodes.back().get();
-  // TODO: object pool
 
-  auto node = new cudaNode(std::forward<ArgsT>(args)...);
-  _nodes.push_back(node);
-  return node;
-}
+  _state |= cudaGraph::CHANGED;
 
-// Procedure: _toposort
-// topological sort iteratively
-inline std::vector<cudaNode*> cudaGraph::_toposort() {
+  auto node = std::make_unique<cudaNode>(std::forward<ArgsT>(args)...);
+  _nodes.emplace_back(std::move(node));
+  return _nodes.back().get();
 
-  std::stack<cudaNode*> dfs;
-  std::vector<cudaNode*> res;
-
-  for(auto node : _nodes) {
-    node->_unset_state(cudaNode::STATE_VISITED);
-  }
-
-  for(auto node : _nodes) {
-    if(!node->_has_state(cudaNode::STATE_VISITED)) {
-      dfs.push(node);
-    }
-
-    while(!dfs.empty()) {
-      auto u = dfs.top();
-      dfs.pop();
-
-      if(u->_has_state(cudaNode::STATE_VISITED)){
-        res.push_back(u);
-        continue;
-      }
-
-      u->_set_state(cudaNode::STATE_VISITED);
-      dfs.push(u);
-
-      for(auto s : u->_successors) {
-        if(!(s->_has_state(cudaNode::STATE_VISITED))) {
-          dfs.push(s);
-        }
-      }
-    }
-  }
-
-  std::reverse(res.begin(), res.end());
-  
-  return res;
+  // TODO: use object pool to save memory
+  //auto node = new cudaNode(std::forward<ArgsT>(args)...);
+  //_nodes.push_back(node);
+  //return node;
 }
 
 // Procedure: dump the graph to a DOT format
 inline void cudaGraph::dump(
   std::ostream& os, const void* root, const std::string& root_name
 ) const {
-  
+
   // recursive dump with stack
   std::stack<std::tuple<const cudaGraph*, const cudaNode*, int>> stack;
   stack.push(std::make_tuple(this, nullptr, 1));
@@ -671,7 +874,7 @@ inline void cudaGraph::dump(
     for(int i=0; i<pl-l+1; i++) {
       os << "}\n";
     }
-  
+
     if(parent == nullptr) {
       if(root) {
         os << "subgraph cluster_p" << root << " {\nlabel=\"cudaFlow: ";
@@ -690,8 +893,10 @@ inline void cudaGraph::dump(
       os << "\";\n" << "color=\"purple\"\n";
     }
 
-    for(auto& v : graph->_nodes) {
-      
+    for(auto& node : graph->_nodes) {
+
+      auto v = node.get();
+
       os << 'p' << v << "[label=\"";
       if(v->_name.empty()) {
         os << 'p' << v << "\"";
@@ -699,18 +904,18 @@ inline void cudaGraph::dump(
       else {
         os << v->_name << "\"";
       }
-          
+
       switch(v->_handle.index()) {
-        case cudaNode::CUDA_KERNEL_TASK:
+        case cudaNode::KERNEL:
           os << " style=\"filled\""
              << " color=\"white\" fillcolor=\"black\""
              << " fontcolor=\"white\""
              << " shape=\"box3d\"";
         break;
 
-        case cudaNode::CUDA_SUBFLOW_TASK:
+        case cudaNode::SUBFLOW:
           stack.push(std::make_tuple(
-            &std::get<cudaNode::Subflow>(v->_handle).graph, v, l+1)
+            &(std::get_if<cudaNode::Subflow>(&v->_handle)->graph), v, l+1)
           );
           os << " style=\"filled\""
              << " color=\"black\" fillcolor=\"purple\""
@@ -721,13 +926,13 @@ inline void cudaGraph::dump(
         default:
         break;
       }
-  
+
       os << "];\n";
 
       for(const auto s : v->_successors) {
         os << 'p' << v << " -> " << 'p' << s << ";\n";
       }
-      
+
       if(v->_successors.size() == 0) {
         if(parent == nullptr) {
           if(root) {
@@ -739,7 +944,7 @@ inline void cudaGraph::dump(
         }
       }
     }
-    
+
     // set the previous level
     pl = l;
   }
