@@ -1,5 +1,5 @@
 /*
- *    Copyright (C) 2016-2022 Grok Image Compression Inc.
+ *    Copyright (C) 2016-2023 Grok Image Compression Inc.
  *
  *    This source code is free software: you can redistribute it and/or  modify
  *    it under the terms of the GNU Affero General Public License, version 3,
@@ -54,7 +54,7 @@ static j2k_prog_order j2k_prog_order_list[] = {{GRK_CPRL, "CPRL"}, {GRK_LRCP, "L
 											   {GRK_PCRL, "PCRL"}, {GRK_RLCP, "RLCP"},
 											   {GRK_RPCL, "RPCL"}, {(GRK_PROG_ORDER)-1, ""}};
 
-CodeStreamCompress::CodeStreamCompress(IBufferedStream* stream) : CodeStream(stream)
+CodeStreamCompress::CodeStreamCompress(BufferedStream* stream) : CodeStream(stream)
 {
 	cp_.wholeTileDecompress_ = false;
 }
@@ -148,11 +148,8 @@ bool CodeStreamCompress::init(grk_cparameters* parameters, GrkImage* image)
 			return false;
 		}
 	}
-
-    if (parameters->apply_icc_){
-        if (image->validateICC())
-            image->applyICC();
-    }
+    if (parameters->apply_icc_)
+       image->applyICC();
 
 	// create private sanitized copy of image
 	headerImage_ = new GrkImage();
@@ -748,7 +745,7 @@ bool CodeStreamCompress::write_eoc()
 
 	return stream_->flush();
 }
-bool CodeStreamCompress::write_mct_record(grk_mct_data* p_mct_record, IBufferedStream* stream)
+bool CodeStreamCompress::write_mct_record(grk_mct_data* p_mct_record, BufferedStream* stream)
 {
 	uint32_t mct_size;
 	uint32_t tmp;
@@ -1331,7 +1328,7 @@ bool CodeStreamCompress::write_regions()
 	return true;
 }
 bool CodeStreamCompress::write_mcc_record(grk_simple_mcc_decorrelation_data* p_mcc_record,
-										  IBufferedStream* stream)
+										  BufferedStream* stream)
 {
 	uint32_t i;
 	uint32_t mcc_size;
