@@ -1,5 +1,5 @@
 /*
- *    Copyright (C) 2016-2022 Grok Image Compression Inc.
+ *    Copyright (C) 2016-2023 Grok Image Compression Inc.
  *
  *    This source code is free software: you can redistribute it and/or  modify
  *    it under the terms of the GNU Affero General Public License, version 3,
@@ -23,7 +23,7 @@
 
 namespace grk
 {
-CodeStreamDecompress::CodeStreamDecompress(IBufferedStream* stream)
+CodeStreamDecompress::CodeStreamDecompress(BufferedStream* stream)
 	: CodeStream(stream), expectSOD_(false), curr_marker_(0), headerError_(false),
 	  headerRead_(false), marker_scratch_(nullptr), marker_scratch_size_(0), outputImage_(nullptr),
 	  tileCache_(new TileCache()), ioBufferCallback(nullptr), ioUserData(nullptr),
