@@ -1,5 +1,5 @@
 /*
- *    Copyright (C) 2016-2022 Grok Image Compression Inc.
+ *    Copyright (C) 2016-2023 Grok Image Compression Inc.
  *
  *    This source code is free software: you can redistribute it and/or  modify
  *    it under the terms of the GNU Affero General Public License, version 3,
@@ -39,7 +39,7 @@ struct BoxWriteHandler
 	uint32_t size_;
 };
 
-FileFormatCompress::FileFormatCompress(IBufferedStream* stream)
+FileFormatCompress::FileFormatCompress(BufferedStream* stream)
 	: FileFormat(), codeStream(new CodeStreamCompress(stream)), needs_xl_jp2c_box_length(false),
 	  j2k_codestream_offset(0), inputImage_(nullptr)
 {}
@@ -787,11 +787,7 @@ bool FileFormatCompress::init(grk_cparameters* parameters, GrkImage* image)
 			comps[i].bpc = (uint8_t)(comps[i].bpc + (1 << 7));
 	}
 
-	if(inputImage_->meta && inputImage_->meta->color.icc_profile_buf)
-	{
-		if(inputImage_->validateICC())
-			inputImage_->color_space = GRK_CLRSPC_ICC;
-	}
+	inputImage_->validateICC();
 
 	/* Colour Specification box */
 	if(inputImage_->color_space == GRK_CLRSPC_ICC)
