@@ -1,5 +1,5 @@
 /*
- *    Copyright (C) 2016-2022 Grok Image Compression Inc.
+ *    Copyright (C) 2016-2023 Grok Image Compression Inc.
  *
  *    This source code is free software: you can redistribute it and/or  modify
  *    it under the terms of the GNU Affero General Public License, version 3,
@@ -23,7 +23,7 @@
 
 namespace grk
 {
-FileFormatDecompress::FileFormatDecompress(IBufferedStream* stream)
+FileFormatDecompress::FileFormatDecompress(BufferedStream* stream)
 	: FileFormat(), headerError_(false), codeStream(new CodeStreamDecompress(stream)), jp2_state(0)
 {
 	header = {{JP2_JP, [this](uint8_t* data, uint32_t len) { return read_jp(data, len); }},
@@ -204,12 +204,8 @@ bool FileFormatDecompress::readHeader(grk_header_info* header_info)
 				image->color_space = GRK_CLRSPC_UNKNOWN;
 				break;
 		}
-		if(meth == 2 && getColour()->icc_profile_buf)
-		{
-			// validate
-			if(image->validateICC())
-				image->color_space = GRK_CLRSPC_ICC;
-		}
+        image->validateICC();
+
 		// check RGB subsampling
 		if(image->color_space == GRK_CLRSPC_SRGB)
 		{
@@ -550,7 +546,7 @@ cleanup:
  *
  */
 bool FileFormatDecompress::read_box_hdr(FileFormatBox* box, uint32_t* p_number_bytes_read,
-										IBufferedStream* stream)
+										BufferedStream* stream)
 {
 	assert(stream != nullptr);
 	assert(box != nullptr);
